.ig
	front.t
..
.de Hl
.br
\l'\\n[.l]u-\\n[.i]u\&\\$1'
.br
..
.ps 10
.vs 12
.nr PS 10
.nr VS 12
.nr SS_prefix 1v
.nr do-page 0
.Nh 0 0
.\" .so utp.mac
.\" .utp
\Z@\D't 8p'@
.Hl
\D't 0'
.sp .6i
.DS R
.ps 52
\m[maroon]GNU/Linux\m[]
.sp 18p
.ps 16
\f[BMB]THE MAN PAGES BOOK\fP
.sp 30p
.sp 1.4i
.ps 12
\f[HB]Maintainers:\fP
.sp 2p
.ps 10
\f[HB]Alejandro Colomar <alx@kernel.org> 2020 - present (5.09 - HEAD)
Michael Kerrisk <mtk.manpages@gmail.com> 2004 - 2021    (2.00 - 5.13)
Andries Brouwer <aeb@cwi.nl> 1995 - 2004     (1.6  - 1.70)
Rik Faith 1993 - 1995     \0(1.0  - 1.5)\fP
.DE
